@@ -1,5 +1,5 @@
 /*
- * $Id: ngrep.c,v 1.14 2001/04/23 14:34:06 jpr5 Exp $
+ * $Id: ngrep.c,v 1.18 2001/06/06 22:07:45 jpr5 Exp $
  *
  * Copyright (c) 2001  Jordan Ritter <jpr5@darkridge.com>
  *
@@ -31,6 +31,7 @@
 #include <getopt.h>
 #include <arpa/inet.h>
 #include <ctype.h>
+#include <time.h>
 #endif
 
 #if defined(AIX)
@@ -62,7 +63,7 @@
 #include "ngrep.h"
 
 
-static char rcsver[] = "$Revision: 1.14 $";
+static char rcsver[] = "$Revision: 1.18 $";
 
 int snaplen = 65535, promisc = 1, to = 1000;
 int show_empty = 0, show_hex = 0, quiet = 0;
@@ -116,7 +117,7 @@ int main(int argc, char **argv) {
   signal(SIGPIPE, clean_exit);
   signal(SIGWINCH, update_windowsize);
 
-  while ((c = getopt(argc, argv, "hXViwqpevxlDtTn:d:A:I:O:")) != EOF) {
+  while ((c = getopt(argc, argv, "hXViwqpevxlDtTs:n:d:A:I:O:")) != EOF) {
     switch (c) {
     case 'I':  
       read_file = optarg;
@@ -133,6 +134,9 @@ int main(int argc, char **argv) {
     case 'n':
       max_matches = atoi(optarg);
       break;
+    case 's':
+      snaplen = atoi(optarg);
+      break;
     case 'T':
       print_time = &print_time_diff_init;
       break;
@@ -428,7 +432,10 @@ void process(u_char *data1, struct pcap_pkthdr* h, u_char *p) {
     }
     
     data = ((char*)tcp) + tcphdr_offset;
-    len = ntohs(ip_packet->ip_len) - ip_hl - tcphdr_offset;
+
+    if ((len = ntohs(ip_packet->ip_len)) < h->caplen)
+      len -= ip_hl + tcphdr_offset;
+    else len = h->caplen - link_offset - ip_hl - tcphdr_offset;
 
     if (((len || show_empty) && (((int)(*match_func)(data, len)) != invert_match))
 	|| keep_matching) { 
@@ -479,7 +486,10 @@ void process(u_char *data1, struct pcap_pkthdr* h, u_char *p) {
     }
 
     data = ((char*)udp) + udphdr_offset;
-    len = ntohs(ip_packet->ip_len) - ip_hl - udphdr_offset;
+
+    if ((len = ntohs(ip_packet->ip_len)) < h->caplen)
+      len -= ip_hl + udphdr_offset;
+    else len = h->caplen - link_offset - ip_hl - udphdr_offset;
 
     if (((len || show_empty) && (((int)(*match_func)(data, len)) != invert_match))
 	|| keep_matching) { 
@@ -528,7 +538,10 @@ void process(u_char *data1, struct pcap_pkthdr* h, u_char *p) {
     }
 
     data = ((char*)ic) + icmphdr_offset;
-    len = ntohs(ip_packet->ip_len) - ip_hl - icmphdr_offset;
+
+    if ((len = ntohs(ip_packet->ip_len)) < h->caplen)
+      len -= ip_hl + icmphdr_offset;
+    else len = h->caplen - link_offset - ip_hl - icmphdr_offset;
 
     if (((len || show_empty) && (((int)(*match_func)(data, len)) != invert_match))
 	|| keep_matching) { 
@@ -790,7 +803,7 @@ void update_windowsize(int e) {
 
 void usage(int e) {
   printf("usage: ngrep <-hXViwqpevxlDtT> <-IO pcap_dump> <-n num> <-d dev> <-A num>\n"
-	 "                               <match expression> <bpf filter>\n");
+	 "                               <-s snaplen> <match expression> <bpf filter>\n");
   exit(e);
 }
 
